package tv.freewheel.ad.interfaces;

import java.util.ArrayList;
import java.util.HashMap;
import android.app.Activity;
import android.view.SurfaceView;
import android.widget.FrameLayout;

/**
 * IAdContext is the main facade for ad requesting, should be created in each Activity that wants to display Ads.
 */
public interface IAdContext {
	
	/**
	 *  Get constants look up table.
	 *  
	 *  @return an IConstants object
	 */
	public IConstants getConstants();

	/**
	 * Set the video display.
	 * <br>
	 * Deprecated. Use {@link #registerVideoDisplayBase(FrameLayout)}registerVideoDisplayBase instead.
	 * <br>
	 *      <strong>Important Note: please put the SurfaceView in a <em>FrameLayout</em>,
	 *              only <em>FrameLayout</em> container is supported currently.</strong>
	 * @param videoDisplay the SurfaceView which resides in a FrameLayout
	 */
	@Deprecated
	public void registerVideoDisplay(SurfaceView videoDisplay);

	/**
	 *	Set the FrameLayout on which the temporal ads will be drawn.
	 *	<br>
	 *	AdManager will use it to find where to play the temporal ads.
	 *	Video ads will fill the FrameLayout and overlay ads will be put at a certain position of the FrameLayout.
	 *  <br>
	 *	
	 *	@param videoDisplayBase the FrameLayout on which the temporal ads will be drawn
	 */
	public void registerVideoDisplayBase(FrameLayout videoDisplayBase);

	/**
	 *  Reset the exclusivity scope. <br>
	 */
	public void resetExclusivity();
	
	/**
	 *	Start a new subsession with subsession token.
	 *
	 *	Following requests will be in the subsession, until this method is called again with a different token.
	 *	It will turn on FW_CAPABILITY_SYNC_MULTI_REQUESTS capability.
	 *
 	 *	@param subsessionToken	a positive int value to identify the subsession
 	 */ 	 
	public void startSubsession(int subsessionToken);	
	
 	/**
	 *  Set the Activity object which create this adContext. <br>
	 *  An Ad Context should be attached to an Activity.
     *
	 *  @param activity the Activity this IAdContext is attached to
	 */
	public void setActivity(Activity activity);
	
	/**
	 *	Return application's current Activity which create and hold the reference of this AdContext.
	 *
	 *	@return the Activity object
	 */
	public Activity getActivity();
	
	/**
	 *  Should be invoked when the Activity's state change.
	 */
	public void setActivityState(int state);
	
	/**
	 *	Set the video play state.
	 *
	 *	@param	videoState the type of the state, the value should be one of:
	 *		<br>
	 *		<br>	VIDEO_STATE_PLAYING
	 *		<br>	VIDEO_STATE_PAUSED
	 *		<br>	VIDEO_STATE_STOPPED
	 *		<br>	VIDEO_STATE_COMPLETED
	 */
	public void setVideoState(int videoState);
	
	/**
	 *	Set the parameters for a special level.
	 *	
	 *	@param name key of the parameter to be set
	 *	@param value value for the key
	 *  @param level level of the parameters, must be one of:
	 *  	<br>
	 *  	<br> PARAMETER_LEVEL_GLOBAL
	 *  	<br> PARAMETER_LEVEL_OVERRIDE
	 */
	public void setParameter(String name, Object value, int level);
	
	/**
	 *  Set the profiles which describes the player and slots.
	 *
	 *  @param  playerProfile                   name of the global profile
	 *  @param  defaultTemporalSlotProfile      name of the temporal slot default profile, null for default
	 *  @param  defaultVideoPlayerSlotProfile   name of the video player slot default profile, null for default
	 *  @param  defaultSiteSectionSlotProfile   name of the site section slot default profile, null for default
	 */
	public void setProfile(String playerProfile, String defaultTemporalSlotProfile, 
			String defaultVideoPlayerSlotProfile, String defaultSiteSectionSlotProfile);
	
	/**
	 *  Set the attributes of the visitor.
	 *
	 *  @param  customId        custom id of the visitor
	 *  @param  ipV4Address     ip address of the visitor
	 *  @param  bandwidth       bandwidth of the visitor
	 *  @param  bandwidthSource bandwidth source of the visitor
	 */
	public void setVisitor(String customId, String ipV4Address, 
							int bandwidth, String bandwidthSource);
	
	/**
	 *  Set the HTTP headers of the visitor.
	 *
	 *  @param  name    name of the header to be set
	 *  @param  value   value of the header to be set, null to remove
	 */
	public void setVisitorHttpHeader(String name, String value);
	
	/**
	 *  Set the capabilities supported by the player.
	 *
	 *  @param  capability name of the capability, should be one of:
	 *  	<br>
	 *  	<br>CAPABILITY_SLOT_TEMPLATE, ON by default
	 *  	<br>CAPABILITY_ADUNIT_IN_MULTIPLE_SLOTS, ON by default
	 *  	<br>CAPABILITY_SLOT_CALLBACK, ON by default
	 *  	<br>CAPABILITY_BYPASS_COMMERCIAL_RATIO_RESTRICTION
	 *  	<br>CAPABILITY_SYNC_MULTI_REQUESTS
	 *  	<br>CAPABILITY_RESET_EXCLUSIVITY
	 *  	<br>CAPABILITY_RECORD_VIDEO_VIEW
	 *  	<br>
	 *  
	 *  @param  status the status be set, should be one of:
	 *  	<br>
	 *  	<br>CAPABILITY_STATUS_ON
	 *  	<br>CAPABILITY_STATUS_OFF
	 *  	<br>CAPABILITY_STATUS_DEFAULT
	 *  	<br>
	 *
	 *  Note: <br>
	 *  Changing renderer capability after renderer starts playing may result undefined behaviour.
	 */
	public void setCapability(String capability, int status);
	
	/**
	 *  Set the attributes of the site section.
	 *
	 *  @param  siteSectionId   id of the site section
	 *  @param  pageViewRandom  unique id for current playing
	 *  @param  networkId       id of the network the site section belongs to, 0 for default
	 *  @param  idType          type of the id above, should be one of
	 *  							<br>
	 *                              <br> ID_TYPE_CUSTOM
	 *                              <br> ID_TYPE_FW
	 *                              <br> ID_TYPE_FWGROUP
	 *  @param  fallbackId      site section id to fallback, if id specified by the 1st parameter is not found, 0 for default
	 */
	public void setSiteSection(String siteSectionId, int pageViewRandom, int networkId, 
								int idType, int fallbackId);
	
    /**
     *  Set the attributes of the video player.
     *  
     *  @param videoPlayerNetworkId  id of the network the player belongs to
     */
	public void setVideoPlayer(int videoPlayerNetworkId);
	
	/**
	 *  Set the attributes of current video asset to play.
	 *
	 *  @param  videoAssetId    id of the video
	 *  @param  duration        duration of the video in seconds
	 *  @param  location        location of the video, null for default
	 *  @param  autoPlay        whether the video begins to play automatically without user interaction
	 *  @param  videoPlayRandom random number that is generated each time a user watches this video asset
	 *  @param  networkId       id of the network the video belongs to, 0 for default
	 *  @param  idType          type of video id above, should be one of:
	 *  							<br>
	 *                              <br> ID_TYPE_CUSTOM
	 *                              <br> ID_TYPE_FW
	 *                              <br> ID_TYPE_FWGROUP
	 *  @param  fallbackId      video id to fallback, if id specified by the 1st parameter is not found, 0 for default
	 */
	public void setVideoAsset(String videoAssetId, double duration, 
			String location, boolean autoPlay, int videoPlayRandom, 
			int networkId, int idType, int fallbackId);
	
	/**
	 *  Set the attributes of current video asset to play.
	 *
	 *  @param  videoAssetId    id of the video
	 *  @param  duration        duration of the video in seconds
	 *  @param  location        location of the video, null for default
	 *  @param  autoPlayType	auto play mode, includes:
	 *  						<ul><li> VIDEO_ASSET_AUTO_PLAY_TYPE_NONE
	 *  							<li> VIDEO_ASSET_AUTO_PLAY_TYPE_ATTENDED
	 *  							<li> VIDEO_ASSET_AUTO_PLAY_TYPE_UNATTENDED</ul>
	 *  							<br>
	 *  @param  videoPlayRandom random number that is generated each time a user watches this video asset.
	 *  @param  networkId       id of the network the video belongs to, 0 for default
	 *  @param  idType          type of video id above, includes:
	 *                          <ul><li> ID_TYPE_CUSTOM
	 *                              <li> ID_TYPE_FW
	 *                              <li> ID_TYPE_FWGROUP</ul>
	 *  							<br>
	 *  @param  fallbackId      video id to fallback, if id specified by the 1st parameter is not found, 0 for default
	 *  @param  durationType    type of duration of asset, includes
	 *  						<ul><li> VIDEO_ASSET_DURATION_TYPE_EXACT
	 *  							<li> VIDEO_ASSET_DURATION_TYPE_VARIABLE</ul>
	 *  							<br> 
	 */
	public void setVideoAsset(String videoAssetId, double duration, 
			String location, int autoPlayType, int videoPlayRandom, 
			int networkId, int idType, int fallbackId, int durationType);
	
	/**
	 * Set time position of video asset.
	 *
	 * If the stream is broken into multiple distinct files, this should be the time position within the asset as a whole.
	 *
	 * @param timePosition	a non-negative number in seconds indicate the time position of video asset  
	 */
	public void setVideoAssetCurrentTimePosition(double timePosition);
	
	/**
	 * Set the mode of request.
	 *
	 *		AdManager runs in On-Demand mode by default.
	 *		To switch to live mode by invocation of this method right after getting a new AdContext instance.
	 *
	 * @param requestMode	the type of request mode
	 * 						<br>
	 * 						<br> REQUEST_MODE_ON_DEMAND
	 * 						<br> REQUEST_MODE_LIVE  
	 */
	public void setRequestMode(int requestMode);
	
	/**
	 * Set duration of request in live mode.
	 *
	 * @param requestDuration	the value of duration in seconds
	 */
	public void setRequestDuration(double requestDuration);

	/**
	 *  Add the key-value pair, if add different values with the same key, they will both the values for the key.
	 *
	 *  @param  key     key name of the key-value pair to be set, null or empty string will no effect
	 *  @param  value   value to be set, null has no effect
	 */
	public void addKeyValue(String key, String value);
	
	/**
	 *  Add a temporal slot.
	 *
	 *  @param  customId                    custom id of the slot, add a slot with an identical name as existing slot will be ignored
	 *  @param  adUnit                      ad unit supported by the slot
	 *  @param  timePosition                time position of the slot
	 *  @param  slotProfile                 profile name of the slot, null for default
	 *  @param  cuePointSequence            sequence of the cuePoint of the slot
	 *  @param  maxDuration                 maximum duration of the slot allowed, 0 for default
	 *  @param  acceptPrimaryContentType    accepted primary content types, use "," as delimiter, null for default
	 *  @param  acceptContentType           accepted content types, use "," as delimiter, null for default
	 *  @param  minDuration                 minimum duration of the slot allowed, 0 for default
	 */
	public void addTemporalSlot(String customId, String adUnit, 
			double timePosition, String slotProfile, int cuePointSequence, 
			double maxDuration, String acceptPrimaryContentType, 
			String acceptContentType, double minDuration);
	
	/**
	 *  Add a video player non-temporal slot.
	 *  
	 *  @param  customId                    custom id of the slot, add a slot with an identical name as existing slot will be ignored
	 *  @param  adUnit                      ad unit supported by the slot
	 *  @param  width                       width of the slot in device-independent pixels
	 *  @param  height                      height of the slot in device-independent pixels
	 *  @param  slotProfile                 profile name of the slot, null for default
	 *  @param  acceptCompanion             whether to accept companion ads on this slots
	 *  @param  acceptPrimaryContentType    accepted primary content types, use "," as delimiter, null for default
	 *  @param  acceptContentType           accepted content types, use "," as delimiter, null for default
	 */
	public void addVideoPlayerNonTemporalSlot(String customId, String adUnit, int width, int height, 
			String slotProfile, boolean acceptCompanion, 
			String acceptPrimaryContentType, String acceptContentType);
	
	/**
	 *  Add a video player non-temporal slot.
	 *  
	 *  @param  customId                    custom id of the slot, add a slot with an identical name as existing slot will be ignored
	 *  @param  adUnit                      ad unit supported by the slot
	 *  @param  width                       width of the slot in device-independent pixels
	 *  @param  height                      height of the slot in device-independent pixels
	 *  @param  slotProfile                 profile name of the slot, null for default
	 *  @param  acceptCompanion             whether to accept companion ads on this slots
	 *  @param  acceptPrimaryContentType    accepted primary content types, use "," as delimiter, null for default
	 *  @param  acceptContentType           accepted content types, use "," as delimiter, null for default
	 *  @param  initialAdOption				choice of the initial ad of this slot, should be one of
	 *		<br>
	 *	 	<br> SLOT_OPTION_INITIAL_AD_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_KEEP_ORIGINAL
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_ONLY
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_OR_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_THEN_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_OR_NO_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_NO_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_NO_STAND_ALONE_IF_TEMPORAL
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_OR_NO_STAND_ALONE_IF_TEMPORAL
	 *  @param  compatibleDimensions		compatible dimensions in form of "width,height|width,height", such as ""320,240|480,720"
	 */
	public void addVideoPlayerNonTemporalSlot(String customId, String adUnit, int width, int height, 
			String slotProfile, boolean acceptCompanion, 
			String acceptPrimaryContentType, String acceptContentType,
			int initialAdOption, String compatibleDimensions);
	
	/**
	 *  Add a site section non-temporal slot.
	 *
	 *  @param  customId                    custom id of the slot, adding a slot with an identical name as existing slot will be ignored
	 *  @param  adUnit                      ad unit supported by the slot
	 *  @param  width                       width of the slot in device-independent pixels
	 *  @param  height                      height of the slot in device-independent pixels
	 *  @param  slotProfile                 profile name of the slot, null for default
	 *  @param  acceptCompanion             whether to accept companion ads on this slots
	 *  @param  acceptPrimaryContentType    accepted primary content types, use "," as delimiter, null for default
	 *  @param  acceptContentType           accepted content types, use "," as delimiter, null for default
	 *  @param  initialAdOption				choice of the initial ad of this slot, should be one of
	 *		<br>
	 *	 	<br> SLOT_OPTION_INITIAL_AD_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_KEEP_ORIGINAL
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_ONLY
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_OR_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_THEN_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_OR_NO_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_NO_STAND_ALONE
	 *		<br> SLOT_OPTION_INITIAL_AD_NO_STAND_ALONE_IF_TEMPORAL
	 *		<br> SLOT_OPTION_INITIAL_AD_FIRST_COMPANION_OR_NO_STAND_ALONE_IF_TEMPORAL
	 *  @param  compatibleDimensions		compatible dimensions in form of "width,height|width,height", such as ""320,240|480,720"
	 */
	public void addSiteSectionNonTemporalSlot(String customId, String adUnit, int width, int height, 
			String slotProfile, boolean acceptCompanion, 
			String acceptPrimaryContentType, String acceptContentType,
			int initialAdOption, String compatibleDimensions);
	
	/**
	 *  Add a site section non-temporal slot.
	 *
	 *  @param  customId                    custom id of the slot, adding a slot with an identical name as existing slot will be ignored
	 *  @param  adUnit                      ad unit supported by the slot
	 *  @param  width                       width of the slot in device-independent pixels
	 *  @param  height                      height of the slot in device-independent pixels
	 *  @param  slotProfile                 profile name of the slot, null for default
	 *  @param  acceptCompanion             whether to accept companion ads on this slots
	 *  @param  acceptPrimaryContentType    accepted primary content types, use "," as delimiter, null for default
	 *  @param  acceptContentType           accepted content types, use "," as delimiter, null for default
	 */
	public void addSiteSectionNonTemporalSlot(String customId, String adUnit, int width, int height, 
			String slotProfile, boolean acceptCompanion, 
			String acceptPrimaryContentType, String acceptContentType);
	
	/**
	 *	Get a slots by its customId.
	 *	
	 *	@param customId custom id of the slot
	 *	@return an ISlot object, or null if not found
	 */
	public ISlot getSlotByCustomId(String customId);
	
	/**
	 *	Get all slots by its timePositionClass.
	 *	
	 *	@param timePositionClass the value should be one of:
	 *		<br>
	 *	 	<br> TIME_POSITION_CLASS_PREROLL
	 *		<br> TIME_POSITION_CLASS_MIDROLL
	 *		<br> TIME_POSITION_CLASS_POSTROLL
	 *		<br> TIME_POSITION_CLASS_OVERLAY
	 *		<br> TIME_POSITION_CLASS_DISPLAY
	 *	    <br> TIME_POSITION_CLASS_PAUSE_MIDROLL
	 *	@return an Array of ISlot object, will return an empty array if none found
	 */
	public ArrayList<ISlot> getSlotsByTimePositionClass(int timePositionClass);
	
	/**
	 *	Get all the temporal slots.
	 *
	 *	@return an Array of ISlot object, will return an empty array if none found
	 */
	public ArrayList<ISlot> getTemporalSlots();
	
	/**
	 *	Get all the video player non-temporal slots.
	 *
	 *	@return an Array of ISlot object, will return an empty array if none found
	 */
	public ArrayList<ISlot> getVideoPlayerNonTemporalSlots();
	
	/**
	 *	Get all the site section non-temporal slots.
	 *
	 *	@return an Array of ISlot object, will return an empty array if none found
	 */
	public ArrayList<ISlot> getSiteSectionNonTemporalSlots();
	
	/**
     *  Submit the request to FreeWheel Ad Server.
	 *
     *  Each AdContext can only submitRequest once.
     *  
     *  @param timeoutSeconds timeout time in seconds for this request
     */
	public void submitRequest(double timeoutSeconds);
	
	public void addRenderer(String url, String baseUnit, String adUnit, String contentType, String slotType, HashMap<String, Object> parameters);
	
	public void addRenderer(String url, String baseUnit, String adUnit, String contentType, String slotType, String creativeAPI, HashMap<String, Object> parameters);
	
	/**
	 * Register an event listener.
	 *
	 * @param type name of the event to listen to
	 * @param listener the listener
	 */
	public void addEventListener(String type, IEventListener listener);
	
	
	/**
	 * Remove an event listener.
	 *
	 * @param type the name of the event to listen to
	 * @param listener the listener
	 */
	public void removeEventListener(String type, IEventListener listener);
	
	public void dispatchEvent(IEvent event);
	
	/**
	 * Dispose this AdContext object and remove all event listeners of it, stop all playing slots.
	 * <br>
	 * After this call, all other methods should not be invoked in this adContext.
	 */
	public void dispose();
	
	/**
	 *	Get video asset's url location.
	 *
	 *	@return a String set by API setVideoAsset's location parameter, or null if not set
	 */
	public String getVideoLocation();
	
	/**
	 *	Retrieve a parameter.
	 *
	 *  @param  name name of the parameter need to retrieve
	 */
	public Object getParameter(String name);
	
	/**
	 *  Get the AdManager object which create the current AdContext.
	 *
	 *  @return an IAdManager object
	 */
	public IAdManager getAdManager();
	
	/**
	 *	Get the mode of ad request.
	 *
	 *	@return A int value set by API setRequestMode , possible value is:
	 *						<br> REQUEST_MODE_ON_DEMAND
	 * 						<br> REQUEST_MODE_LIVE 
	 *			<br> default value is REQUEST_MODE_ON_DEMAND
	 */
	public int getRequestMode();
	
	/**
	 *	Set the user actions.
	 *
	 *	@param userAction The value should be one of:
	 *						<br> USER_ACTION_PAUSE_BUTTON_CLICKED
	 *						<br> USER_ACTION_RESUME_BUTTON_CLICKED
	 */
	public void notifyUserAction(int userAction);
	
	/**
	 *	Load player extension from given URL which implements IExtension.
	 *
	 *	<br>Event IConstants.EVENT_EXTENSION_LOADED will be fired when an extension succeeds or fails to initialize.
	 *	
	 *	@param name a String identifier of the extension
	 */
	public void loadExtension(String name);
}
