package tv.freewheel.ad.interfaces;

import java.util.ArrayList;

/**
 *	Internal context of an ad instance.
 *
 *	Note:
 *		Some methods and references to ICreativeRendition and ICreativeRenditionAsset
 *		are *not* available for player right now.
 */
public interface IAdInstance {
	/**
	 * Get the ad id.
	 *
	 * @return an int value represent ad id
	 */
	public int getAdId();

	/**
	 * Get IRendererController of the ad. Only for player use.
	 *
	 * @return IRendererController of the ad
	 */
	public IRendererController getRendererController();

	/**
	 * Retrieve the parameters from the ad instance.
	 *
	 * The parameter value will be retrieved from levels in the following order:
	 * override, creative rendition, creative, slot, profile, global.
	 * Only for player use.
	 *
	 * @param    name    parameter name
	 * @return the value of the parameter
	 */
	public Object getParameter(String name);

	/**
	 *	Get the ad slot which contain this ad. Only for renderer use.
	 *
	 *	@return an object of ISlot
	 */
	public ISlot getSlot();

 	/**
	 *	Get the duration of the ad. It returns the most accurate value available to AdManager, which may come from the renderer or the ad response XML. 
	 * 	The value reported by the renderer is considered more accurate than the value from ad response, aka. the creative duration value booked in MRM UI.
	 *
	 *	@return the duration of the ad, in seconds.
	 */
	public double getDuration();

	/**
	 *	Get the playhead time of the ad.
	 *
	 *	@return the playhead time of the ad, in seconds.
	 */
	public double getPlayheadTime();


	/**
	 *	Get active rendition of the ad instance. Only for renderer use.
	 *
	 *	@return an object of ICreativeRendition
	 */
	public ICreativeRendition getActiveCreativeRendition();

	/**
	 *	Get all renditions of this ad instance. Only for renderer use.
	 *
	 *	@return an ArrayList of type ICreativeRendition
	 */
	public ArrayList<ICreativeRendition> getAllCreativeRenditions();

	/**
	 * Get renderable renditions of this ad instance. Only for renderer use.
	 *
	 * @return an ArrayList of renderable ICreativeRendition
	 */
	public ArrayList<ICreativeRendition> getRenderableCreativeRenditions();

	/**
	 *	Set the active rendition of this ad, this rendition will finally be showed. Only for renderer use.
	 *
	 *	@param rendition the active rendition will be set
	 */
	public void setActiveCreativeRendition(ICreativeRendition rendition);

	/**
	 * Add a creative rendition to the ad instance. Only for renderer use.
	 *
	 * @return the ICreativeRendition object added to the ad instance
	 */
	public ICreativeRendition createCreativeRenditionForTranslation();

	/**
	 * Append additional tracking URLs to certain event specified by eventName and eventType. Only for renderer use.
	 *
	 * @param eventName name of the event
	 * @param eventType type of the event
	 * @param urls ArrayList of URLs to be added
	 * <br>
	 * <br>
	 * Available combination of input:<br>
	 *		(EVENT_AD_IMPRESSION, EVENT_TYPE_IMPRESSION)	-	impression, aka ad display start<br>
	 *		(EVENT_AD_CLICK,      EVENT_TYPE_CLICK_TRACKING)	-	click tracking<br>
	 *		("a_custom_string",   EVENT_TYPE_CLICK_TRACKING)	-	custom click tracking<br>
	 *		(EVENT_AD_FIRST_QUARTILE,     EVENT_TYPE_IMPRESSION)	-	1st quartile<br>
	 *		(EVENTCALLBACK_MIDPOINT,      EVENT_TYPE_IMPRESSION)	-	middle point<br>
	 *		(EVENTCALLBACK_THIRDQUARTILE, EVENT_TYPE_IMPRESSION)	-	3rd quartile<br>
	 *		(EVENTCALLBACK_COMPLETE,      EVENT_TYPE_IMPRESSION)	-	complete<br>
	 *		(EVENT_AD_PAUSE,              EVENT_TYPE_STANDARD)		-	IAB metric, pause<br>
	 *		(EVENT_AD_RESUME,             EVENT_TYPE_STANDARD)		-	IAB metric, resume<br>
	 *		(EVENT_AD_REWIND,             EVENT_TYPE_STANDARD)		-	IAB metric, rewind<br>
	 *		(EVENT_AD_MUTE,               EVENT_TYPE_STANDARD)		-	IAB metric, mute<br>
	 *		(EVENT_AD_UNMUTE,             EVENT_TYPE_STANDARD)		-	IAB metric, unmute<br>
	 *		(EVENT_AD_COLLAPSE,           EVENT_TYPE_STANDARD)		-	IAB metric, collapse<br>
	 *		(EVENT_AD_EXPAND,             EVENT_TYPE_STANDARD)		-	IAB metric, expand<br>
	 *		(EVENT_AD_MINIMIZE,           EVENT_TYPE_STANDARD)		-	IAB metric, minimize<br>
	 *		(EVENT_AD_CLOSE,              EVENT_TYPE_STANDARD)		-	IAB metric, close<br>
	 *		(EVENT_AD_ACCEPT_INVITATION,  EVENT_TYPE_STANDARD)		-	IAB metric, accept invitation<br>
	 */
	public void addEventCallbackURLs(String eventName, String eventType, final ArrayList<String> urls);

	/**
	 * Set click through URL (replaces the CR value). Only for renderer use.
	 *
	 * @param url URL in String
	 * @param clickName name of the click in String, if clickName is null or empty, will set click through url to default click callback
	 */
	public void setClickThroughURL(String url, String clickName);

	/**
	 * Get the event callback URLs in an ArrayList. Only for renderer use.
	 *
	 * @param eventName name of the event
	 * @param eventType type of the event
	 * @return ArrayList of event callback URLs
	 * <br>
	 * <br>
	 * Available combination of input:<br>
	 *		(EVENT_AD_IMPRESSION, EVENT_TYPE_IMPRESSION)	-	impression, aka ad display start<br>
	 *		(EVENT_AD_CLICK,      EVENT_TYPE_CLICK)			-	click through<br>
	 *		(EVENT_AD_CLICK,      EVENT_TYPE_CLICK_TRACKING)	-	click tracking<br>
	 *		("a_custom_string",   EVENT_TYPE_CLICK)			-	custom click<br>
	 *		("a_custom_string",   EVENT_TYPE_CLICK_TRACKING)	-	custom click tracking<br>
	 *		(EVENT_AD_FIRST_QUARTILE,     EVENT_TYPE_IMPRESSION)	-	1st quartile<br>
	 *		(EVENTCALLBACK_MIDPOINT,      EVENT_TYPE_IMPRESSION)	-	middle point<br>
	 *		(EVENTCALLBACK_THIRDQUARTILE, EVENT_TYPE_IMPRESSION)	-	3rd quartile<br>
	 *		(EVENTCALLBACK_COMPLETE,      EVENT_TYPE_IMPRESSION)	-	complete<br>
	 *		(EVENT_AD_PAUSE,              EVENT_TYPE_STANDARD)		-	IAB metric, pause<br>
	 *		(EVENT_AD_RESUME,             EVENT_TYPE_STANDARD)		-	IAB metric, resume<br>
	 *		(EVENT_AD_REWIND,             EVENT_TYPE_STANDARD)		-	IAB metric, rewind<br>
	 *		(EVENT_AD_MUTE,               EVENT_TYPE_STANDARD)		-	IAB metric, mute<br>
	 *		(EVENT_AD_UNMUTE,             EVENT_TYPE_STANDARD)		-	IAB metric, unmute<br>
	 *		(EVENT_AD_COLLAPSE,           EVENT_TYPE_STANDARD)		-	IAB metric, collapse<br>
	 *		(EVENT_AD_EXPAND,             EVENT_TYPE_STANDARD)		-	IAB metric, expand<br>
	 *		(EVENT_AD_MINIMIZE,           EVENT_TYPE_STANDARD)		-	IAB metric, minimize<br>
	 *		(EVENT_AD_CLOSE,              EVENT_TYPE_STANDARD)		-	IAB metric, close<br>
	 *		(EVENT_AD_ACCEPT_INVITATION,  EVENT_TYPE_STANDARD)		-	IAB metric, accept invitation<br>
	 */
	public ArrayList<String> getEventCallbackURLs(String eventName, String eventType);
	
	/**
	 *	Get the playable companion ad instances.
	 *	
	 *	@return an ArrayList of IAdInstance instances
	 */
	public ArrayList<IAdInstance> getCompanionAdInstances();
	
	/**
	 *	Whether the ad is required to be shown by law, usually the ad is a companion ad.
	 *	
	 *	@return true if this ad is required to be shown, otherwise false
	 */
	public boolean isRequiredToShow();
}
