package tv.freewheel.ad.interfaces;

import android.view.ViewGroup;
import java.util.ArrayList;

/**
 * ISlot is the base Ad placement unit, which is used to play the ads.
 */
public interface ISlot {
	/**
	 *	Get custom id of the slot.
	 *
	 *	@return custom id of the slot
	 */
	public String getCustomId();
	
	/**
	 *	Get type of the slot.
	 *
	 *	@return type of the slot, the value can be one of:
	 *		<p>	SLOT_TYPE_TEMPORAL
	 *		<p>	SLOT_TYPE_VIDEOPLAYER_NONTEMPORAL
	 *		<p>	SLOT_TYPE_SITESECTION_NONTEMPORAL
	 */
	public int getType();
	
	/**
	 *	Get time position of the slot.
	 *
	 *	@return time position of the slot
	 */
	public double getTimePosition();
	
	/**
	 *	Get time position class.
	 *
	 *	@return time position class of the slot, the value can be one of:
	 *		<p>	TIME_POSITION_CLASS_PREROLL
	 *		<p>	TIME_POSITION_CLASS_MIDROLL
	 *		<p>	TIME_POSITION_CLASS_POSTROLL
	 *		<p>	TIME_POSITION_CLASS_OVERLAY
	 *		<p>	TIME_POSITION_CLASS_DISPLAY
	 *		<p> TIME_POSITION_CLASS_PAUSE_MIDROLL
	 */
	public int getTimePositionClass();

	/**
	 *	Get the duration of the embedded ads.
	 *
	 *	@return the duration of embedded ads, -1 if unavailable
	 */
	public double getEmbeddedAdsDuration();

	/**
	 *  Get the end time position of the slot.
	 *
	 *  @return the end time position of the temporal slot, -1 if unavailable
	 */
	public double getEndTimePosition();
	
	/**
	 *	Get width of the slot.
	 *
	 *	@return width in device-independent pixels
	 */
	public int getWidth();
	
	/**
	 *	Get height of the slot.
	 *
	 *	@return height in device-independent pixels
	 */
	public int getHeight();
	
	/**
	 *	Get slot display base object, the ad will be drawn on the base. <br>
	 *
	 *	@return the ViewGroup object. For Temporal Slot it is a FrameLayout, for nonTemporal Slot it is a RelativeLayout. If failed to create slotBase will return null
	 */
	public ViewGroup getBase();
	
	/**
	 *	Play the slot.
	 */
	public void play();
	
	/**
	 * 	Preload the slot.
	 */
	public void preload();
	
	/**
	 *	Stop the slot.
	 */
	public void stop();
	
	/**
	 * 	Pause the slot.
	 */
	public void pause();
	
	/**
	 * 	Resume the slot.
	 */
	public void resume();
	
	/**
	 *	Set the parameters at slot level.
	 *
	 *	@param	name	key of the parameter to be set
	 *	@param	value	value for the key
	 */
	public void setParameter(String name, Object value);
	
	/**
	 *	Get duration of the slot.
	 *
	 *	@return a number in seconds, which is greater than or equal to 0
	 */
	public double getTotalDuration();
	
	/**
	 *	Get playheadTime of the slot.
	 *
	 *	@return a number in seconds, which is greater than or equal to 0
	 */
	public double getPlayheadTime();

	/**
	 * Get all ad instances in the slot. Only for player use.
	 *
	 * @return ArrayList of ad instances
	 */
	public ArrayList<IAdInstance> getAdInstances();
}
