package tv.freewheel.renderers.interfaces;

import java.util.HashMap;
import java.util.ArrayList;

import android.widget.FrameLayout;
import tv.freewheel.ad.interfaces.IAdInstance;
import tv.freewheel.ad.interfaces.IConstants;
import tv.freewheel.ad.interfaces.ISlot;
import android.app.Activity;
import android.location.Location;

/**
 *	Interface for a renderer to obtain information from AdManager.
 */
public interface IRendererContext {
	/**
	 *	Get current ad instance.
	 *	
	 *	@return an IAdInstance
	 *		
	 */
	public IAdInstance getAdInstance();

	/**
	 *	Retrieve a parameter.
	 *  
	 *  @param name name of the parameter need to retrieve
	 */
	public Object getParameter(String name);

	/**
	 *  Get constants look up table.
	 *  
	 *  @return an IConstants object
	 */
	public IConstants getConstants();
	
	/**
	 *	Return application's current Activity which create and hold the reference of this AdContext.
	 *
	 *	@return the Activity object
	 */
	public Activity getActivity();
	
	/**
	 *	Declare capability of the renderer.
	 *
	 *	@param	rendererCapability one of EVENT_AD_* (NOT including EVENT_AD_FIRST_QUARTILE, EVENT_AD_THIRD_QUARTILE, EVENT_AD_IMPRESSION) in IConstants
	 *	@param	status the status be set, should be one of:
	 *  	<br>
	 *  	<br>CAPABILITY_STATUS_ON
	 *  	<br>CAPABILITY_STATUS_OFF
	 *  	<br>CAPABILITY_STATUS_DEFAULT
	 *  	<br>
	 *	Note: <br>
	 *	Changing renderer capability after renderer starts playing may result undefined behaviour.
	 */
	public void setRendererCapability(String rendererCapability, int status);
	
	/**
	 *	Dispatch renderer event.
	 *
	 *	@param	eventName event to be processed, one of EVENT_AD_* and EVENT_REQUEST_CONTENT_VIDEO_* in IConstants
	 */
	public void dispatchEvent(String eventName);
	
	/**
	 *	Dispatch renderer event.
	 *
	 *	@param	eventName event to be processed, one of EVENT_AD_* and EVENT_REQUEST_CONTENT_VIDEO_* in IConstants
	 *  @param  extraInfo additional information need to be processed by AdManager. Available keys:
	 * 					<br>- INFO_KEY_ERROR_CODE Optional. 
	 * 					<br>When eventName is EVENT_ERROR, use this key to specify the error code.
	 * 					<br>- INFO_KEY_ERROR_INFO Optional. 
	 * 					<br>When eventName is EVENT_ERROR, use this key to specify the description of the error.
	 * 					<br>- INFO_KEY_SHOW_BROWSER Optional.
	 * 					<br>Only valid when the eventName is EVENT_AD_CLICK, the value should be boolean
	 * 					<br>use this key to specify whether the click event should open a browser if possible
	 * 					<br>the default value is true
	 * 					<br>- INFO_KEY_URL Optional.
	 * 					<br>Only valid when the eventName is EVENT_AD_CLICK, the value should be String.
	 * 					<br>use this key to specify redirect URL.
	 * 					<br>the default value is the one specified on MRM UI.
	 * 					<br>- INFO_KEY_MESSAGE Optional.
	 * 					<br>When eventName is EVENT_CLICK, use this key to specify the name of custom click
	 */
	public void dispatchEvent(String eventName, HashMap<String, Object> extraInfo);
	
	/**
	 *	Get the location of device. This value was set by IAdManager.setLocation and will not be change in the AdContext's lifetime.
	 *
	 *	@return	the location value which App set through IAdManager.setLocation. will be null if not set
	 */
	public Location getLocation();
	
	/**
	 * Register event callback listener for Activity's callback event.
	 * 
	 * supported events:
	 * 			<br>ACTIVITY_CALLBACK_PAUSE
	 * 			<br>ACTIVITY_CALLBACK_RESUME
	 * 
	 * @param listener the event callback listener to response for the event
	 */
	public void addOnActivityCallbackListener(IActivityCallbackListener listener);
	
	/**
	 * Schedule ad instances for slots passed in as parameter. Usually slots consists of current slot and its companion slots.
	 *
	 * @param slots slots to be scheduled for ads
	 * @return ArrayList of scheduled IAdInstance
	 */
	public ArrayList<IAdInstance> scheduleAdInstances(ArrayList<ISlot> slots);
	
	/**
	 * Return the companion slots which are able to schedule companion.
	 *
	 * @return ArrayList of companion ISlot
	 */
	public ArrayList<ISlot> getCompanionSlots();
	
	/**
	 * Get SDK version.
	 *
	 * @return Current version as int, e.g., 0x02060000 for v2.6
	 */
	public int getVersion();
}
